///////////////////////////////////////////
//
// file:  MIX_application.cpp
//
// Copyright 2004 John R. Ashmun
//                jrashmun@halcyon.com
//
//    This file is part of MIX. 
//
//    MIX is free software; you can redistribute it and/or modify 
//    it under the terms of the GNU General Public License as
//    published by the Free Software Foundation; either version 2
//    of the License, or (at your option) any later version. 
//
//    MIX is distributed in the hope that it will be useful, 
//    but WITHOUT ANY WARRANTY; without even the implied warranty of 
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
//    GNU General Public License for more details. 
//
//    You should have received a copy of the GNU General Public
//    License along with MIX; if not, write to the
//    Free Software Foundation, Inc.
//    59 Temple Place, Suite 330
//    Boston MA 02111-1307  USA
//
///////////////////////////////////////////


#include <iostream>

#include "MIX.h"
#include "MIX_application.h"


typedef void (*op_func)( );

op_func opcodes[64] = 
{
   NOP,  ADD,  SUB,  MUL,  DIV,  SPEC, SHIFT, MOVE,
   LD,   LD,   LD,   LD,   LD,   LD,   LD,    LD,
   LD,   LD,   LD,   LD,   LD,   LD,   LD,    LD,
   ST,   ST,   ST,   ST,   ST,   ST,   ST,    ST,
   ST,   ST,   JBUS, IOC,  IN,   OUT,  JRED,  JCMP,
   JBIT, JBIT, JBIT, JBIT, JBIT, JBIT, JBIT,  JBIT,
   INC,  INC,  INC,  INC,  INC,  INC,  INC,   INC,
   CMP,  CMP,  CMP,  CMP,  CMP,  CMP,  CMP,   CMP
};

MIX_application::MIX_application( )
   : BApplication( "application/x-vnd.jbg-MIX" )
{
}


MIX_application::~MIX_application( )
{
}


void MIX_application::ReadyToRun( )
{
   bool          clock_interrupt = false;
   bool          was_normal_state = false;

   int           i = 0;
int clock_interrupts_count = 0;
   int           last_saved_u_counter = 0;
   int           opcode = 0;
   int           u_count = 0;

   MIX_Go_button Go_button;

   // Initialize the device objects
   // by setting their unit numbers
   // (Each will set its block size
   // and set its state to "ready".)
   for( i = 0; i < 20; i++ )
      DEVICES[i].unit( i );

   program_counter = Go_button.push( );
/*
u_count = 100;
cs_MEMORY[10].Byte_0( u_count >> 31 );
cs_MEMORY[10].Byte_1( (u_count >> 24) & 0x3F );
cs_MEMORY[10].Byte_2( (u_count >> 18) & 0x3F );
cs_MEMORY[10].Byte_3( (u_count >> 12) & 0x3F );
cs_MEMORY[10].Byte_4( (u_count >> 6) & 0x3F );
cs_MEMORY[10].Byte_5( u_count & 0x3F );
*/
   while( cycle )
   {
      if( !control_state )
      {
         if( program_counter < 0 || program_counter > 3999 )
         {
            cerr << "ERROR -- attempt to address unavailable memory" << endl;
            cycle = false;
            break;
         }
      }
      else
      {
         if( program_counter < -3999 || program_counter > 3999 )
         {
            cerr << "ERROR -- attempt to address unavailable memory" << endl;
            cycle = false;
            break;
         }
      }

//cerr << "program_counter == " << program_counter << " ";
      if( !control_state )
      {
         was_normal_state = true;
         INST = MEMORY[program_counter++];
      }
      else
      {
         was_normal_state = false;

         if( program_counter < 0 )
            INST = cs_MEMORY[-program_counter--];
         else
            INST = MEMORY[program_counter++];
      }

      opcode = INST.Byte_5( );
//cerr << "opcode == " << opcode << endl;

snooze( 1 );

      opcodes[opcode]( );

      if( (u_counter - last_saved_u_counter) >= 1000 )
      {
         last_saved_u_counter = u_counter;

         u_count = ((cs_MEMORY[10].Byte_0( ) << 31)
                    | (cs_MEMORY[10].Byte_1( ) << 24)
                    | (cs_MEMORY[10].Byte_2( ) << 18)
                    | (cs_MEMORY[10].Byte_3( ) << 12)
                    | (cs_MEMORY[10].Byte_4( ) << 6)
                    | cs_MEMORY[10].Byte_5( )) - 1;

         cs_MEMORY[10].Byte_0( u_count >> 31 );
         cs_MEMORY[10].Byte_1( (u_count >> 24) & 0x3F );
         cs_MEMORY[10].Byte_2( (u_count >> 18) & 0x3F );
         cs_MEMORY[10].Byte_3( (u_count >> 12) & 0x3F );
         cs_MEMORY[10].Byte_4( (u_count >> 6) & 0x3F );
         cs_MEMORY[10].Byte_5( u_count & 0x3F );

         if( u_count == 0 )
            clock_interrupt = true;
      }

      if( !control_state && was_normal_state )
      {
         if( clock_interrupt )
         {
            clock_interrupt = false;

clock_interrupts_count++;
/*
u_count = 100;
cs_MEMORY[10].Byte_0( u_count >> 31 );
cs_MEMORY[10].Byte_1( (u_count >> 24) & 0x3F );
cs_MEMORY[10].Byte_2( (u_count >> 18) & 0x3F );
cs_MEMORY[10].Byte_3( (u_count >> 12) & 0x3F );
cs_MEMORY[10].Byte_4( (u_count >> 6) & 0x3F );
cs_MEMORY[10].Byte_5( u_count & 0x3F );
*/
            enter_control_state( );
            program_counter = -11;
         }
         else
         {
            for( i = 0; i < 20; i++ )
            {
               if( DEVICES[i].interrupt( ) )
               {
                  enter_control_state( );
                  program_counter = -(i + 20);
                  break;
               }
            }
         }
      }
   }

   cerr << "Total time:  " << u_counter << " u." << endl;
cerr << "Number of clock interrupts:  " << clock_interrupts_count << "." << endl;

   BMessage quit( B_QUIT_REQUESTED );
   be_app->PostMessage( &quit );
}
