///////////////////////////////////////////
//
// file:  MIX_word.cpp
//
// Copyright 2004 John R. Ashmun
//                jrashmun@halcyon.com
//
//    This file is part of MIX. 
//
//    MIX is free software; you can redistribute it and/or modify 
//    it under the terms of the GNU General Public License as
//    published by the Free Software Foundation; either version 2
//    of the License, or (at your option) any later version. 
//
//    MIX is distributed in the hope that it will be useful, 
//    but WITHOUT ANY WARRANTY; without even the implied warranty of 
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
//    GNU General Public License for more details. 
//
//    You should have received a copy of the GNU General Public
//    License along with MIX; if not, write to the
//    Free Software Foundation, Inc.
//    59 Temple Place, Suite 330
//    Boston MA 02111-1307  USA
//
///////////////////////////////////////////


#include "MIX.h"


MIX_word::MIX_word( )
{
   _value._int_value = 0;
}


MIX_word::~MIX_word( )
{
}


MIX_word::MIX_word( const MIX_word& M_w )
{
   _value._int_value = M_w._value._int_value;
}


MIX_word& MIX_word::operator=( const MIX_word& M_w )
{
   _value._int_value = M_w._value._int_value;

   return *this;
}


void MIX_word::A( const MIX_word& A )
{
   Byte_0( A.Byte_0( ) );
   Byte_1( A.Byte_1( ) );
   Byte_2( A.Byte_2( ) );
}


void MIX_word::F( const int L, const int R )
{
   Byte_4( 8 * L + R );
}


void MIX_word::I( const int index )
{
   Byte_3( index );
}


void MIX_word::F( const int field )
{
   Byte_4( field );
}


void MIX_word::C( const int opcode )
{
   Byte_5( opcode );
}


void MIX_word::Byte_0( const int i )
{
   _value._int_value = (_value._int_value & 0x7FFFFFFF) | ((i & 1) << 31);
}


void MIX_word::Byte_1( const int i )
{
   _value._int_value = (_value._int_value & 0xC0FFFFFF) | ((i & 0x3F) << 24);
}


void MIX_word::Byte_2( const int i )
{
   _value._int_value = (_value._int_value & 0xFF03FFFF) | ((i & 0x3F) << 18);
}


void MIX_word::Byte_3( const int i )
{
   _value._int_value = (_value._int_value & 0xFFFC0FFF) | ((i & 0x3F) << 12);
}


void MIX_word::Byte_4( const int i )
{
   _value._int_value = (_value._int_value & 0xFFFFF03F) | ((i & 0x3F) << 6);
}


void MIX_word::Byte_5( const int i )
{
   _value._int_value = (_value._int_value & 0xFFFFFFC0) | (i & 0x3F);
}


int MIX_word::A( ) const
{
   int retval = ((Byte_1( ) << 6) | Byte_2( ));

   if( Byte_0( ) )
      retval = -retval;

   return retval;
}


int MIX_word::I( ) const
{
   return Byte_3( );
}


int MIX_word::F( ) const
{
   return Byte_4( );
}


int MIX_word::C( ) const
{
   return Byte_5( );
}


int MIX_word::M( ) const
{
   int index_reg = I( );
   int retval = A( );

   if( index_reg > 0 && index_reg <= 6 )
   {
      switch( index_reg )
      {
         case 1:
            retval += I1REG.A( );
            break;

         case 2:
            retval += I2REG.A( );
            break;

         case 3:
            retval += I3REG.A( );
            break;

         case 4:
            retval += I4REG.A( );
            break;

         case 5:
            retval += I5REG.A( );
            break;

         case 6:
            retval += I6REG.A( );
            break;

         default:
            break;
      }
   }

   return retval;
}


int MIX_word::Byte_0( ) const
{
   return ((_value._int_value >> 31) & 1);
}


int MIX_word::Byte_1( )const
{
   return ((_value._int_value >> 24) & 0x3F);
}


int MIX_word::Byte_2( )const
{
   return ((_value._int_value >> 18) & 0x3F);
}


int MIX_word::Byte_3( )const
{
   return ((_value._int_value >> 12) & 0x3F);
}


int MIX_word::Byte_4( )const
{
   return ((_value._int_value >> 6) & 0x3F);
}


int MIX_word::Byte_5( )const
{
   return (_value._int_value & 0x3F);
}


void MIX_word::int_value( const int v )
{
   _value._int_value = v;
}


int MIX_word::int_value( ) const
{
   return _value._int_value;
}
